﻿using System.Text;
using System.Web.Mvc;

namespace DS.Web.Mvc.PathHelper
{
    /// <summary>
    /// Hmtl扩展
    /// </summary>
    public static class HtmlExtensions
    {
        /// <summary>
        /// 得到网站标题
        /// </summary>
        /// <param name="html"></param>
        /// <returns></returns>
        public static MvcHtmlString GetTitle(this HtmlHelper html)
        {
            return GetWebPath(html, "{0}", " « ", true);
        }

        /// <summary>
        /// 得到网站路径
        /// </summary>
        /// <param name="html"></param>
        /// <returns></returns>
        public static MvcHtmlString GetSitemap(this HtmlHelper html)
        {
            return GetWebPath(html, "<a title=\"{0}\" href=\"{1}\">{0}</a>", " » ");
        }

        /// <summary>
        /// 得到通用路径,标题用{0}代替，链接用{1}代替
        /// </summary>
        /// <param name="html"></param>
        /// <param name="itemTemplate">项目模板</param>
        /// <param name="separatorTemplate">间隔符模板</param>
        /// <param name="desc">是否倒序</param>
        /// <returns></returns>
        public static MvcHtmlString GetWebPath(this HtmlHelper html, string itemTemplate, string separatorTemplate,bool desc = false)
        {

            var sb = new StringBuilder();
            var webPath = (WebPath) (html.ViewData[WebPath.DefaultName] ?? new WebPath());

            if (desc)
            {
                //倒序
                for (var k = webPath.Count - 1; k > -1; k--)
                {
                    if (sb.Length != 0) sb.Append(separatorTemplate);
                    sb.Append(itemTemplate.Replace("{0}", webPath[k][0]).Replace("{1}", webPath[k][1]));
                }
            }
            else
            {
                //正序
                foreach (var p in webPath)
                {
                    if (sb.Length != 0) sb.Append(separatorTemplate);
                    sb.Append(itemTemplate.Replace("{0}", p[0]).Replace("{1}", p[1]));
                }
            }
            return MvcHtmlString.Create(sb.ToString());
        }
    }
}
