﻿using System;
using System.Web.Mvc;
using System.Web.Routing;

namespace DS.Web.Mvc.PathHelper
{
    /// <summary>
    /// 路由设置特征
    /// </summary>
    [AttributeUsage(AttributeTargets.Class|AttributeTargets.Method,AllowMultiple=true)]
    public class NewPathAttribute :ActionFilterAttribute
    {
        private readonly string _name;
        private readonly string[] _routevalues;

        /// <summary>
        /// Area
        /// </summary>
        public string Area { get; set; }
        /// <summary>
        /// Controller
        /// </summary>
        public string Controller { get; set; }
        /// <summary>
        /// Action
        /// </summary>
        public string Action { get; set; }
        /// <summary>
        /// Id
        /// </summary>
        public string Id { get; set; }

        /// <summary>
        /// 设置新路径
        /// </summary>
        /// <param name="name"></param>
        public NewPathAttribute(string name)
        {
            _name = name;
            Order = int.MaxValue;
        }
        /// <summary>
        /// 设置新路径
        /// </summary>
        /// <param name="name"></param>
        /// <param name="routevalues"></param>
        public NewPathAttribute(string name, params string[] routevalues)
        {
            _name = name;
            _routevalues = routevalues;
            Order = int.MaxValue;
        }

        /// <summary>
        /// Action执行前执行
        /// </summary>
        /// <param name="filterContext"></param>
        public override void OnActionExecuted(ActionExecutedContext filterContext)
        {

        }

        /// <summary>
        /// Action执行前执行
        /// </summary>
        /// <param name="filterContext"></param>
        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            var viewData = filterContext.Controller.ViewData;
            var webPath = (WebPath) (viewData[WebPath.DefaultName] ?? (viewData[WebPath.DefaultName] = new WebPath()));
            webPath.AddPath(_name, GetPath(filterContext));
        }

        /// <summary>
        /// 得到路径
        /// </summary>
        /// <param name="filterContext"></param>
        /// <returns></returns>
        private string GetPath(ActionExecutingContext filterContext)
        {
            var rd = new RouteValueDictionary();
            if (!string.IsNullOrEmpty(Area)) rd.Add("Area", Area);
            if (!string.IsNullOrEmpty(Controller)) rd.Add("Controller", Controller);
            if (!string.IsNullOrEmpty(Action)) rd.Add("Action", Action);
            if (!string.IsNullOrEmpty(Id)) rd.Add("Id", Id);
            if (_routevalues != null)
            {
                for (var k = 0; k < _routevalues.Length; k += 2)
                {
                    if (!string.IsNullOrEmpty(Area) && _routevalues[k].ToLower() == "area") continue;
                    if (!string.IsNullOrEmpty(Controller) && _routevalues[k].ToLower() == "controller") continue;
                    if (!string.IsNullOrEmpty(Action) && _routevalues[k].ToLower() == "action") continue;
                    if (!string.IsNullOrEmpty(Id) && _routevalues[k].ToLower() == "id") continue;
                    rd.Add(_routevalues[k], _routevalues[k + 1]);
                }
            }
            var url = new UrlHelper(filterContext.RequestContext);
            return url.RouteUrl(rd);
        }
    }

}